import pygame
import random
from pygame.locals import *

# Initialisatie van Pygame
pygame.init()

# (Schermgrootte) deze functie maakt heyt speelveld waar de slang in kan bewegen dat is opgemaakyt uit cellen dioe in een rooster zijn gezet.
WIDTH, HEIGHT = 500, 500
CELL_SIZE = 20
GRID_WIDTH = WIDTH // CELL_SIZE
GRID_HEIGHT = HEIGHT // CELL_SIZE
SCREEN = pygame.display.set_mode((WIDTH, HEIGHT))
pygame.display.set_caption("Snake Game")

# (Kleuren) dit is een tripple en gaat de kleuren rood groen en blauw mengen om de kleuren te krijgen die in de game zichtbaar zijn. de maximale cocentratie van een kleur is 225 en laagste 0 zo kan je de juiste verhouding van elke kleur in de tripple zetten
WHITE = (255, 255, 255)
BLACK = (0, 0, 0)
RED = (255, 0, 0)
GREEN = (0, 255, 0)
GREY = (40, 40, 40)

# (Richtingen) deze functie gaat er voor zorgen dat de slang naar boven, onder, links en rechts kan gaan om richting de appel te gaan.
UP = (0, -1)
DOWN = (0, 1)
LEFT = (-1, 0)
RIGHT = (1, 0)
# Snake-klasse
class Snake:
    def __init__(self):
        self.body = [(GRID_WIDTH // 2, GRID_HEIGHT // 2)]  #snake in het midden van het veld laten starten
        self.direction = random.choice([UP, DOWN, LEFT, RIGHT])  #random beweegrichtingen bepalen
        self.grow_pending = 0  #zorgen dat de snake niet zelf groeit
    
    def move(self):  #snake in huidige richting verplaatsen
        head = self.body[0]     #hoofd krijgt positie van de slang
        x, y = self.direction   #x en y implementeren op huidige positie
        new_head = (head[0] + x, head[1] + y)  #nieuwe positie hoofd berekenen vie verplaatsing x en y
        if (
            new_head in self.body[1:] or  #controleren of snake gebotst is
            new_head[0] < 0 or new_head[0] >= GRID_WIDTH or
            new_head[1] < 0 or new_head[1] >= GRID_HEIGHT
        ):
            return False   #segmenten aan snake toevoegen (list)
        self.body.insert(0, new_head)
        if self.grow_pending == 0:
            self.body.pop()  # slang beweegt door het laatste element van de slang te verwijderen
        else:    #bij geen botsing true
            self.grow_pending -= 1
        return True
    
    def grow(self):   #groeien bij volgende beweging
        self.grow_pending += 1
    
    def draw(self):  #snake tekenen met pygame
        for segment in self.body:
            pygame.draw.rect(SCREEN, GREEN, (segment[0] * CELL_SIZE, segment[1] * CELL_SIZE, CELL_SIZE, CELL_SIZE))


# Apple-klasse
class Apple:
    def __init__(self):
        self.position = self.random_position()  #appels random positioneren
    
    def random_position(self):
        return (random.randint(0, GRID_WIDTH - 1), random.randint(0, GRID_HEIGHT - 1))
    
    def draw(self):  #appel tekenen via pygame
        pygame.draw.rect(SCREEN, RED, (self.position[0] * CELL_SIZE, self.position[1] * CELL_SIZE, CELL_SIZE, CELL_SIZE))

 
# Functie om het raster te tekenen
def draw_grid():    #dit is een functie die rasters maakt
    for x in range(0, WIDTH, CELL_SIZE):           #dit is de functie die er voor zorgt dat de roosters even groot zijn als de cel die de slang heeft voor die een appel heeft gegeten. de roosters zijn afgebakent met grijze lijnen.
        pygame.draw.line(SCREEN, GREY, (x, 0), (x, HEIGHT))
    for y in range(0, HEIGHT, CELL_SIZE):
        pygame.draw.line(SCREEN, GREY, (0, y), (WIDTH, y))

# Hoofdlus van het spel
def main():
    # hier worden de appel en de slang gemaakt (instance) 
    snake = Snake()
    apple = Apple()
    running = True  
    while running:
        for event in pygame.event.get():
            # de event.type KEYDOWN betekent dat er een toets is ingedrukt
            if event.type == KEYDOWN:
                # de event.key bevat informatie over welke toets is ingedrukt
                # hier worden de slang gestuurd. (hij kan niet terug, dus die beweging moeten we negeren)
                if event.key == K_UP and snake.direction != DOWN:
                    snake.direction = UP
                elif event.key == K_DOWN and snake.direction != UP:
                    snake.direction = DOWN
                elif event.key == K_LEFT and snake.direction != RIGHT:
                    snake.direction = LEFT
                elif event.key == K_RIGHT and snake.direction != LEFT:
                    snake.direction = RIGHT
        # als de slang niet beweegt (botsing) dan stopt het spel
        if not snake.move():
            running = False

        # als de slang de appel raakt, dan groeit die bij de volgende beweging. (hij krijgt een nieuwe kop)
        if snake.body[0] == apple.position:
            snake.grow()
            # verplaats de appel naar een nieuwe random locatie
            apple.position = apple.random_position()

        SCREEN.fill(BLACK)
        draw_grid()
        snake.draw()
        apple.draw()
        pygame.display.flip()
        # hier wordt de snelheid van het spel bepaalt en met pygame's 'draw' wordt bepaalt wat er zichtbaar is in de grid
        pygame.time.Clock().tick(10)
    pygame.quit()

main()

# bron chat gpt en https://dev.to/emilossola/step-by-step-guide-to-build-python-snake-game-with-pygame-4bp
    
